@@ -7,11 +7,11 @@ use Scalar::Util qw( refaddr );
 use List::Util qw( shuffle );
 use File::Spec ();
 use Storable ();
-use TheSchwartz::Moosified::Utils qw/insert_id sql_for_unixtime bind_param_attr run_in_txn order_by_priority/;
+use TheSchwartz::Moosified::Utils qw/insert_id sql_for_unixtime/;
 use TheSchwartz::Moosified::Job;
 use TheSchwartz::Moosified::JobHandle;
 
-our $VERSION = '0.05_005';
+our $VERSION = '0.01';
 our $AUTHORITY = 'cpan:FAYLAND';
 
 ## Number of jobs to fetch at a time in find_job_for_workers.
@@ -23,10 +23,10 @@ my $sub_verbose = sub {
     $msg =~ s/\s+$//;
     print STDERR "$msg\n";
 };
-subtype 'TheSchwartz.Moosified.Verbose'
+subtype 'Verbose'
     => as 'CodeRef'
     => where { 1; };
-coerce 'TheSchwartz.Moosified.Verbose'
+coerce 'Verbose'
     => from 'Int'
     => via {
         if ($_) {
@@ -36,7 +36,7 @@ coerce 'TheSchwartz.Moosified.Verbose'
         }
     };
 
-has 'verbose' => ( is => 'rw', isa => 'TheSchwartz.Moosified.Verbose', coerce => 1, default => 0 );
+has 'verbose' => ( is => 'rw', isa => 'Verbose', coerce => 1, default => 0 );
 has 'prioritize' => ( is => 'rw', isa => 'Bool', default => 0 );
 
 has 'retry_seconds' => (is => 'rw', isa => 'Int', default => 30);
@@ -57,11 +57,25 @@ has 'funcmap_cache' => ( is => 'rw', isa => 'HashRef', default => sub { {} } );
 has 'scoreboard'  => (
     is => 'rw',
     isa => 'Str',
-    trigger => \&_trigger_scoreboard,
-);
+    trigger => sub {
+        my ($self, $dir) = @_;
+        
+        return unless $dir;
+        return if (-f $dir); # no endless loop
 
-has 'prefix' => ( is => 'rw', isa => 'Str', default => '' );
-has 'error_length' => ( is => 'rw', isa => 'Int', default => 255 );
+        # They want the scoreboard but don't care where it goes
+        if (($dir eq '1') or ($dir eq 'on')) {
+            $dir = File::Spec->tmpdir();
+        }
+    
+        $dir .= '/theschwartz';
+        unless (-e $dir) {
+            mkdir($dir, 0755) or die "Can't create scoreboard directory '$dir': $!";
+        }
+        
+        $self->{scoreboard} = $dir."/scoreboard.$$";
+    }
+);
 
 sub debug {
     my $self = shift;
@@ -75,42 +89,6 @@ sub shuffled_databases {
     return shuffle( @{ $self->databases } );
 }
 
-sub _try_insert {
-    my $self = shift;
-    my $job = shift;
-    my $dbh = shift;
-
-    $job->funcid( $self->funcname_to_id( $dbh, $job->funcname ) );
-
-    run_in_txn {
-        $job->insert_time(time());
-
-        my $row = $job->as_hashref;
-        if ($dbh->{Driver}{Name} && $dbh->{Driver}{Name} eq 'Pg') {
-            delete $row->{jobid};
-        }
-        my @col = keys %$row;
-
-        my $table_job = $self->prefix . 'job';
-        my $sql = sprintf 'INSERT INTO %s (%s) VALUES (%s)',
-            $table_job, join( ", ", @col ), join( ", ", ("?") x @col );
-
-        my $sth = $dbh->prepare_cached($sql);
-        my $i = 1;
-        for my $col (@col) {
-            $sth->bind_param(
-                $i++,
-                $row->{$col},
-                bind_param_attr( $dbh, $col ),
-            );
-        }
-        $sth->execute();
-
-        my $jobid = insert_id( $dbh, $sth, $table_job, "jobid" );
-        $job->jobid($jobid);
-    } $dbh;
-}
-
 sub insert {
     my $self = shift;
 
@@ -124,22 +102,35 @@ sub insert {
     $job->arg( Storable::nfreeze( $job->arg ) ) if ref $job->arg;
 
     for my $dbh ( $self->shuffled_databases ) {
+        my $jobid;
         eval {
-            $self->_try_insert($job,$dbh);
+            $job->funcid( $self->funcname_to_id( $dbh, $job->funcname ) );
+            $job->insert_time(time);
+
+            my $row = $job->as_hashref;
+            my @col = keys %$row;
+
+            my $sql = sprintf 'INSERT INTO job (%s) VALUES (%s)',
+                join( ", ", @col ), join( ", ", ("?") x @col );
+
+            my $sth = $dbh->prepare_cached($sql);
+            $sth->execute( @$row{@col} );
+
+            $jobid = insert_id( $dbh, $sth, "job", "jobid" );
+            $job->jobid($jobid);
         };
-        $self->debug("insert failed: $@") if $@;
 
-        next unless $job->jobid;
-
-        ## We inserted the job successfully!
-        ## Attach a handle to the job, and return the handle.
-        my $handle = TheSchwartz::Moosified::JobHandle->new({
-                dbh    => $dbh,
-                client => $self,
-                jobid  => $job->jobid
-            });
-        $job->handle($handle);
-        return $handle;
+        if ($job->jobid) {
+            ## We inserted the job successfully!
+            ## Attach a handle to the job, and return the handle.
+            my $handle = TheSchwartz::Moosified::JobHandle->new({
+                    dbh    => $dbh,
+                    client => $self,
+                    jobid  => $job->jobid
+                });
+            $job->handle($handle);
+            return $handle;
+        }
     }
 
     return;
@@ -154,11 +145,11 @@ sub find_job_for_workers {
     return unless (scalar @$worker_classes);
 
     my $limit    = $FIND_JOB_BATCH_SIZE;
+    my $order_by = $client->prioritize ? 'ORDER BY priority DESC' : '';
 
     for my $dbh ( $client->shuffled_databases ) {
 
         my $unixtime = sql_for_unixtime($dbh);
-        my $order_by = $client->prioritize ? order_by_priority($dbh) : '';
 
         my @jobs;
         eval {
@@ -171,8 +162,7 @@ sub find_job_for_workers {
                       @$worker_classes;
 
             my $ids = join(',', @ids);
-            my $table_job = $client->prefix . 'job';
-            my $sql = qq~SELECT * FROM $table_job WHERE funcid IN ($ids) AND run_after <= $unixtime AND grabbed_until <= $unixtime $order_by LIMIT $limit~;
+            my $sql = qq~SELECT * FROM job WHERE funcid IN ($ids) AND run_after <= $unixtime AND grabbed_until <= $unixtime $order_by LIMIT 0, $limit~;
 
             my $sth = $dbh->prepare_cached($sql);
             $sth->execute();
@@ -180,7 +170,6 @@ sub find_job_for_workers {
                 my $job = TheSchwartz::Moosified::Job->new( $ref );
                 push @jobs, $job;
             }
-            $sth->finish;
         };
 #        if ($@) {
 #
@@ -218,19 +207,15 @@ sub _grab_a_job {
         
         my $new_grabbed_until = $server_time + ($worker_class->grab_for || 1);
 
-        # Prevent a condition that could result in more than one client
-        # grabbing the same job b/c it doesn't look grabbed
-        next JOB if ($new_grabbed_until == $old_grabbed_until);
-
-        ## Update the job in the database, and end the transaction.
-
-        my $table_job = $client->prefix . 'job';
-        my $sql  = qq~UPDATE $table_job SET grabbed_until = ? WHERE jobid = ? AND grabbed_until = ?~;
+        my $sql  = q~UPDATE job SET grabbed_until = ? WHERE jobid = ?~;
         my $sth  = $dbh->prepare($sql);
-        $sth->execute($new_grabbed_until, $job->jobid, $old_grabbed_until);
+        $sth->execute($new_grabbed_until, $job->jobid);
         my $rows = $sth->rows;
 
-        next JOB unless $rows == 1;
+        ## Update the job in the database, and end the transaction.
+        if ($rows < 1) {
+            next JOB;
+        }
         
         $job->grabbed_until( $new_grabbed_until );
 
@@ -273,12 +258,11 @@ sub list_jobs {
         };
     }
     
-    my $limit = $arg->{limit} || $FIND_JOB_BATCH_SIZE;
+    my $limit    = $arg->{limit} || $FIND_JOB_BATCH_SIZE;
+    my $order_by = $self->prioritize ? 'ORDER BY priority DESC' : '';
 
     my @jobs;
     for my $dbh ( $self->shuffled_databases ) {
-        my $order_by = $self->prioritize ? order_by_priority($dbh) : '';
-
         eval {
             
             my ($funcid, $funcop);
@@ -290,19 +274,16 @@ sub list_jobs {
                 $funcop = '=';
             }
 
-            my $table_job = $self->prefix . 'job';
-            my $sql = qq~SELECT * FROM $table_job WHERE funcid $funcop $funcid~;
+            my $sql = qq~SELECT * FROM job WHERE funcid $funcop $funcid $order_by LIMIT 0, $limit~;
             my @value = ();
             for (@options) {
                 $sql .= " AND $_->{key} $_->{op} ?";
                 push @value, $_->{value};
             }
-            $sql .= qq~ $order_by LIMIT $limit~;
 
             my $sth = $dbh->prepare_cached($sql);
             $sth->execute(@value);
             while ( my $ref = $sth->fetchrow_hashref ) {
-                $ref->{funcname} = $self->funcid_to_name($dbh, $ref->{funcid});
                 my $job = TheSchwartz::Moosified::Job->new( $ref );
                 if ($arg->{want_handle}) {
                     my $handle = TheSchwartz::Moosified::JobHandle->new({
@@ -314,7 +295,6 @@ sub list_jobs {
                 }
                 push @jobs, $job;
             }
-            $sth->finish;
         };
     }
 
@@ -341,7 +321,7 @@ sub _find_job_with_coalescing {
     my $order_by = $client->prioritize ? 'ORDER BY priority DESC' : '';
 
     for my $dbh ( $client->shuffled_databases ) {
-        my $unixtime = sql_for_unixtime($dbh);
+        my $unixtime = sql_for_unixtime();
 
         my @jobs;
         eval {
@@ -352,16 +332,13 @@ sub _find_job_with_coalescing {
             ##    in the past).
             my $funcid = $client->funcname_to_id($dbh, $funcname);
 
-            my $table_job = $client->prefix . 'job';
-            my $sql = qq~SELECT * FROM $table_job WHERE funcid = ? AND run_after <= $unixtime AND grabbed_until <= $unixtime AND coalesce $op ? $order_by LIMIT $limit~;
+            my $sql = qq~SELECT * FROM job WHERE funcid = ? AND run_after <= $unixtime AND grabbed_until <= $unixtime AND coalesce $op ? $order_by LIMIT 0, $limit~;
             my $sth = $dbh->prepare_cached($sql);
             $sth->execute( $funcid, $coval );
             while ( my $ref = $sth->fetchrow_hashref ) {
-                $ref->{funcname} = $funcname;
                 my $job = TheSchwartz::Moosified::Job->new( $ref );
                 push @jobs, $job;
             }
-            $sth->finish;
         };
 #        if ($@) {
 #
@@ -470,29 +447,22 @@ sub funcname_to_id {
 
     my $dbid  = refaddr $dbh;
     my $cache = $self->_funcmap_cache($dbh);
-    my $table_funcmap = $self->prefix . 'funcmap';
 
     unless ( exists $cache->{funcname2id}{$funcname} ) {
-        my $id;
-        eval {
-            run_in_txn {
-                ## This might fail in a race condition since funcname is UNIQUE
-                my $sth = $dbh->prepare_cached(
-                    "INSERT INTO $table_funcmap (funcname) VALUES (?)");
-                $sth->execute($funcname);
+        ## This might fail in a race condition since funcname is UNIQUE
+        my $sth = $dbh->prepare_cached(
+            'INSERT INTO funcmap (funcname) VALUES (?)');
+        eval { $sth->execute($funcname) };
 
-                $id = insert_id( $dbh, $sth, $table_funcmap, "funcid" );
-            } $dbh;
-        };
+        my $id = insert_id( $dbh, $sth, "funcmap", "funcid" );
 
         ## If we got an exception, try to load the record again
         if ($@) {
             my $sth = $dbh->prepare_cached(
-                "SELECT funcid FROM $table_funcmap WHERE funcname = ?");
+                'SELECT funcid FROM funcmap WHERE funcname = ?');
             $sth->execute($funcname);
-            ($id) = $sth->fetchrow_array;
-            $sth->finish;
-            croak "Can't find or create funcname $funcname: $@" unless $id;
+            $id = $sth->fetchrow_arrayref->[0]
+                or croak "Can't find or create funcname $funcname: $@";
         }
 
         $cache->{funcname2id}{ $funcname } = $id;
@@ -506,35 +476,19 @@ sub funcname_to_id {
 sub _funcmap_cache {
     my ( $client, $dbh ) = @_;
     my $dbid = refaddr $dbh;
-    my $table_funcmap = $client->prefix . 'funcmap';
     unless ( exists $client->funcmap_cache->{$dbid} ) {
         my $cache = { funcname2id => {}, funcid2name => {} };
-        my $sth = $dbh->prepare_cached("SELECT funcid, funcname FROM $table_funcmap");
+        my $sth = $dbh->prepare_cached('SELECT funcid, funcname FROM funcmap');
         $sth->execute;
         while ( my $row = $sth->fetchrow_arrayref ) {
             $cache->{funcname2id}{ $row->[1] } = $row->[0];
             $cache->{funcid2name}{ $row->[0] } = $row->[1];
         }
-        $sth->finish;
         $client->funcmap_cache->{$dbid} = $cache;
     }
     return $client->funcmap_cache->{$dbid};
 }
 
-sub _trigger_scoreboard {
-    my ($self, $dir) = @_;
-    
-    return unless $dir;
-    return if (-f $dir); # no endless loop
-
-    # They want the scoreboard but don't care where it goes
-    if (($dir eq '1') or ($dir eq 'on')) {
-        $dir = File::Spec->tmpdir();
-    }
-
-    $self->{scoreboard} = $dir."/theschwartz.scoreboard.$$";
-}
-
 sub start_scoreboard {
     my $client = shift;
 
@@ -675,7 +629,7 @@ read more on L<TheSchwartz>
 
 =item * C<databases>
 
-Databases containing TheSchwartz jobs, shuffled before each use.
+An arrayref of dbh.
 
     my $dbh1 = DBI->conncet(@dbi_info);
     my $dbh2 = $schema->storage->dbh;
@@ -687,7 +641,7 @@ Databases containing TheSchwartz jobs, shuffled before each use.
 
 =item * C<verbose>
 
-controls debug logging.
+control the debug.
 
     my $client = TheSchwartz::Moosified->new( verbose => 1 );
     
@@ -699,12 +653,6 @@ controls debug logging.
         print STDERR "[INFO] $msg\n";
     } );
 
-=item * C<prefix>
-
-optional prefix for tables. compatible with L<TheSchwartz::Simple>
-
-    my $client = TheSchwartz::Moosified->new( prefix => 'theschwartz_' );
-
 =item * C<scoreboard>
 
 save job info to file. by default, the file will be saved at $tmpdir/theschwartz/scoreboard.$$
@@ -716,12 +664,6 @@ save job info to file. by default, the file will be saved at $tmpdir/theschwartz
     # be sure the file is there
     $client->scoreboard( "/home/fayland/theschwartz/scoreboard.log" );
 
-=item * C<error_length>
-
-optional, defaults to 255.  Messages logged to the C<failure_log> (the
-C<error> table) are truncated to this length.  Setting this to zero means no
-truncation (although the database you are using may truncate this for you).
-
 =back
 
 =head1 POSTING JOBS
@@ -782,16 +724,10 @@ Note the C<TheSchwartz> implementation of this function uses a C<LIKE> query to
 find matching jobs, with all the attendant performance implications for your
 job databases.
 
-=head1 SEE ALSO
-
-L<TheSchwartz>, L<TheSchwartz::Simple>
-
 =head1 AUTHOR
 
 Fayland Lam, C<< <fayland at gmail.com> >>
 
-Jeremy Stashewsky, C<< <jstash+cpan at gmail.com> >>
-
 =head1 COPYRIGHT & LICENSE
 
 Copyright 2008 Fayland Lam, all rights reserved.
